/****************************************************************************
** Meta object code from reading C++ file 'SpectacleConfig.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/SpectacleConfig.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SpectacleConfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_SpectacleConfig_t {
    QByteArrayData data[41];
    char stringdata0[774];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SpectacleConfig_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SpectacleConfig_t qt_meta_stringdata_SpectacleConfig = {
    {
QT_MOC_LITERAL(0, 0, 15), // "SpectacleConfig"
QT_MOC_LITERAL(1, 16, 18), // "lastSaveAsLocation"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 21), // "setLastSaveAsLocation"
QT_MOC_LITERAL(4, 58, 8), // "location"
QT_MOC_LITERAL(5, 67, 10), // "cropRegion"
QT_MOC_LITERAL(6, 78, 13), // "setCropRegion"
QT_MOC_LITERAL(7, 92, 6), // "region"
QT_MOC_LITERAL(8, 99, 14), // "onClickChecked"
QT_MOC_LITERAL(9, 114, 17), // "setOnClickChecked"
QT_MOC_LITERAL(10, 132, 7), // "enabled"
QT_MOC_LITERAL(11, 140, 21), // "includePointerChecked"
QT_MOC_LITERAL(12, 162, 24), // "setIncludePointerChecked"
QT_MOC_LITERAL(13, 187, 25), // "includeDecorationsChecked"
QT_MOC_LITERAL(14, 213, 28), // "setIncludeDecorationsChecked"
QT_MOC_LITERAL(15, 242, 33), // "captureTransientWindowOnlyChe..."
QT_MOC_LITERAL(16, 276, 36), // "setCaptureTransientWindowOnly..."
QT_MOC_LITERAL(17, 313, 12), // "captureDelay"
QT_MOC_LITERAL(18, 326, 15), // "setCaptureDelay"
QT_MOC_LITERAL(19, 342, 5), // "delay"
QT_MOC_LITERAL(20, 348, 11), // "captureMode"
QT_MOC_LITERAL(21, 360, 14), // "setCaptureMode"
QT_MOC_LITERAL(22, 375, 5), // "index"
QT_MOC_LITERAL(23, 381, 20), // "useDynamicSaveButton"
QT_MOC_LITERAL(24, 402, 23), // "setUseDynamicSaveButton"
QT_MOC_LITERAL(25, 426, 29), // "rememberLastRectangularRegion"
QT_MOC_LITERAL(26, 456, 32), // "setRememberLastRectangularRegion"
QT_MOC_LITERAL(27, 489, 24), // "useLightRegionMaskColour"
QT_MOC_LITERAL(28, 514, 27), // "setUseLightRegionMaskColour"
QT_MOC_LITERAL(29, 542, 16), // "lastUsedSaveMode"
QT_MOC_LITERAL(30, 559, 19), // "setLastUsedSaveMode"
QT_MOC_LITERAL(31, 579, 22), // "autoSaveFilenameFormat"
QT_MOC_LITERAL(32, 602, 25), // "setAutoSaveFilenameFormat"
QT_MOC_LITERAL(33, 628, 6), // "format"
QT_MOC_LITERAL(34, 635, 16), // "autoSaveLocation"
QT_MOC_LITERAL(35, 652, 19), // "setAutoSaveLocation"
QT_MOC_LITERAL(36, 672, 27), // "copySaveLocationToClipboard"
QT_MOC_LITERAL(37, 700, 30), // "setCopySaveLocationToClipboard"
QT_MOC_LITERAL(38, 731, 15), // "saveImageFormat"
QT_MOC_LITERAL(39, 747, 18), // "setSaveImageFormat"
QT_MOC_LITERAL(40, 766, 7) // "saveFmt"

    },
    "SpectacleConfig\0lastSaveAsLocation\0\0"
    "setLastSaveAsLocation\0location\0"
    "cropRegion\0setCropRegion\0region\0"
    "onClickChecked\0setOnClickChecked\0"
    "enabled\0includePointerChecked\0"
    "setIncludePointerChecked\0"
    "includeDecorationsChecked\0"
    "setIncludeDecorationsChecked\0"
    "captureTransientWindowOnlyChecked\0"
    "setCaptureTransientWindowOnlyChecked\0"
    "captureDelay\0setCaptureDelay\0delay\0"
    "captureMode\0setCaptureMode\0index\0"
    "useDynamicSaveButton\0setUseDynamicSaveButton\0"
    "rememberLastRectangularRegion\0"
    "setRememberLastRectangularRegion\0"
    "useLightRegionMaskColour\0"
    "setUseLightRegionMaskColour\0"
    "lastUsedSaveMode\0setLastUsedSaveMode\0"
    "autoSaveFilenameFormat\0setAutoSaveFilenameFormat\0"
    "format\0autoSaveLocation\0setAutoSaveLocation\0"
    "copySaveLocationToClipboard\0"
    "setCopySaveLocationToClipboard\0"
    "saveImageFormat\0setSaveImageFormat\0"
    "saveFmt"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SpectacleConfig[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      32,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  174,    2, 0x0a /* Public */,
       3,    1,  175,    2, 0x0a /* Public */,
       5,    0,  178,    2, 0x0a /* Public */,
       6,    1,  179,    2, 0x0a /* Public */,
       8,    0,  182,    2, 0x0a /* Public */,
       9,    1,  183,    2, 0x0a /* Public */,
      11,    0,  186,    2, 0x0a /* Public */,
      12,    1,  187,    2, 0x0a /* Public */,
      13,    0,  190,    2, 0x0a /* Public */,
      14,    1,  191,    2, 0x0a /* Public */,
      15,    0,  194,    2, 0x0a /* Public */,
      16,    1,  195,    2, 0x0a /* Public */,
      17,    0,  198,    2, 0x0a /* Public */,
      18,    1,  199,    2, 0x0a /* Public */,
      20,    0,  202,    2, 0x0a /* Public */,
      21,    1,  203,    2, 0x0a /* Public */,
      23,    0,  206,    2, 0x0a /* Public */,
      24,    1,  207,    2, 0x0a /* Public */,
      25,    0,  210,    2, 0x0a /* Public */,
      26,    1,  211,    2, 0x0a /* Public */,
      27,    0,  214,    2, 0x0a /* Public */,
      28,    1,  215,    2, 0x0a /* Public */,
      29,    0,  218,    2, 0x0a /* Public */,
      30,    1,  219,    2, 0x0a /* Public */,
      31,    0,  222,    2, 0x0a /* Public */,
      32,    1,  223,    2, 0x0a /* Public */,
      34,    0,  226,    2, 0x0a /* Public */,
      35,    1,  227,    2, 0x0a /* Public */,
      36,    0,  230,    2, 0x0a /* Public */,
      37,    1,  231,    2, 0x0a /* Public */,
      38,    0,  234,    2, 0x0a /* Public */,
      39,    1,  235,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::QUrl,
    QMetaType::Void, QMetaType::QUrl,    4,
    QMetaType::QRect,
    QMetaType::Void, QMetaType::QRect,    7,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::QReal,
    QMetaType::Void, QMetaType::QReal,   19,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,   33,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,   40,

       0        // eod
};

void SpectacleConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        SpectacleConfig *_t = static_cast<SpectacleConfig *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: { QUrl _r = _t->lastSaveAsLocation();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = _r; }  break;
        case 1: _t->setLastSaveAsLocation((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 2: { QRect _r = _t->cropRegion();
            if (_a[0]) *reinterpret_cast< QRect*>(_a[0]) = _r; }  break;
        case 3: _t->setCropRegion((*reinterpret_cast< const QRect(*)>(_a[1]))); break;
        case 4: { bool _r = _t->onClickChecked();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 5: _t->setOnClickChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: { bool _r = _t->includePointerChecked();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 7: _t->setIncludePointerChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: { bool _r = _t->includeDecorationsChecked();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 9: _t->setIncludeDecorationsChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: { bool _r = _t->captureTransientWindowOnlyChecked();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 11: _t->setCaptureTransientWindowOnlyChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: { qreal _r = _t->captureDelay();
            if (_a[0]) *reinterpret_cast< qreal*>(_a[0]) = _r; }  break;
        case 13: _t->setCaptureDelay((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 14: { int _r = _t->captureMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        case 15: _t->setCaptureMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: { bool _r = _t->useDynamicSaveButton();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 17: _t->setUseDynamicSaveButton((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: { bool _r = _t->rememberLastRectangularRegion();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 19: _t->setRememberLastRectangularRegion((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: { bool _r = _t->useLightRegionMaskColour();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 21: _t->setUseLightRegionMaskColour((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: { int _r = _t->lastUsedSaveMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        case 23: _t->setLastUsedSaveMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: { QString _r = _t->autoSaveFilenameFormat();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 25: _t->setAutoSaveFilenameFormat((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: { QString _r = _t->autoSaveLocation();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 27: _t->setAutoSaveLocation((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: { bool _r = _t->copySaveLocationToClipboard();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 29: _t->setCopySaveLocationToClipboard((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: { QString _r = _t->saveImageFormat();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 31: _t->setSaveImageFormat((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject SpectacleConfig::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_SpectacleConfig.data,
      qt_meta_data_SpectacleConfig,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *SpectacleConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpectacleConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_SpectacleConfig.stringdata0))
        return static_cast<void*>(const_cast< SpectacleConfig*>(this));
    return QObject::qt_metacast(_clname);
}

int SpectacleConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 32;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
