/****************************************************************************
** Meta object code from reading C++ file 'ExportManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/ExportManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ExportManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_ExportManager_t {
    QByteArrayData data[25];
    char stringdata0[257];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportManager_t qt_meta_stringdata_ExportManager = {
    {
QT_MOC_LITERAL(0, 0, 13), // "ExportManager"
QT_MOC_LITERAL(1, 14, 12), // "errorMessage"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 3), // "str"
QT_MOC_LITERAL(4, 32, 19), // "saveLocationChanged"
QT_MOC_LITERAL(5, 52, 8), // "location"
QT_MOC_LITERAL(6, 61, 13), // "pixmapChanged"
QT_MOC_LITERAL(7, 75, 6), // "pixmap"
QT_MOC_LITERAL(8, 82, 10), // "imageSaved"
QT_MOC_LITERAL(9, 93, 7), // "savedAt"
QT_MOC_LITERAL(10, 101, 11), // "forceNotify"
QT_MOC_LITERAL(11, 113, 19), // "getAutosaveFilename"
QT_MOC_LITERAL(12, 133, 8), // "tempSave"
QT_MOC_LITERAL(13, 142, 8), // "mimetype"
QT_MOC_LITERAL(14, 151, 6), // "doSave"
QT_MOC_LITERAL(15, 158, 3), // "url"
QT_MOC_LITERAL(16, 162, 6), // "notify"
QT_MOC_LITERAL(17, 169, 8), // "doSaveAs"
QT_MOC_LITERAL(18, 178, 8), // "QWidget*"
QT_MOC_LITERAL(19, 187, 12), // "parentWindow"
QT_MOC_LITERAL(20, 200, 17), // "doCopyToClipboard"
QT_MOC_LITERAL(21, 218, 7), // "doPrint"
QT_MOC_LITERAL(22, 226, 9), // "QPrinter*"
QT_MOC_LITERAL(23, 236, 7), // "printer"
QT_MOC_LITERAL(24, 244, 12) // "saveLocation"

    },
    "ExportManager\0errorMessage\0\0str\0"
    "saveLocationChanged\0location\0pixmapChanged\0"
    "pixmap\0imageSaved\0savedAt\0forceNotify\0"
    "getAutosaveFilename\0tempSave\0mimetype\0"
    "doSave\0url\0notify\0doSaveAs\0QWidget*\0"
    "parentWindow\0doCopyToClipboard\0doPrint\0"
    "QPrinter*\0printer\0saveLocation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportManager[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       2,  126, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       6,    1,   95,    2, 0x06 /* Public */,
       8,    1,   98,    2, 0x06 /* Public */,
      10,    1,  101,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,  104,    2, 0x0a /* Public */,
      12,    1,  105,    2, 0x0a /* Public */,
      12,    0,  108,    2, 0x2a /* Public | MethodCloned */,
      14,    2,  109,    2, 0x0a /* Public */,
      14,    1,  114,    2, 0x2a /* Public | MethodCloned */,
      14,    0,  117,    2, 0x2a /* Public | MethodCloned */,
      17,    1,  118,    2, 0x0a /* Public */,
      17,    0,  121,    2, 0x2a /* Public | MethodCloned */,
      20,    0,  122,    2, 0x0a /* Public */,
      21,    1,  123,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QPixmap,    7,
    QMetaType::Void, QMetaType::QUrl,    9,
    QMetaType::Void, QMetaType::QUrl,    9,

 // slots: parameters
    QMetaType::QUrl,
    QMetaType::QUrl, QMetaType::QString,   13,
    QMetaType::QUrl,
    QMetaType::Void, QMetaType::QUrl, QMetaType::Bool,   15,   16,
    QMetaType::Void, QMetaType::QUrl,   15,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 22,   23,

 // properties: name, type, flags
      24, QMetaType::QString, 0x00495103,
       7, QMetaType::QPixmap, 0x00495103,

 // properties: notify_signal_id
       1,
       2,

       0        // eod
};

void ExportManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        ExportManager *_t = static_cast<ExportManager *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->errorMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->saveLocationChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->pixmapChanged((*reinterpret_cast< const QPixmap(*)>(_a[1]))); break;
        case 3: _t->imageSaved((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 4: _t->forceNotify((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: { QUrl _r = _t->getAutosaveFilename();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = _r; }  break;
        case 6: { QUrl _r = _t->tempSave((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = _r; }  break;
        case 7: { QUrl _r = _t->tempSave();
            if (_a[0]) *reinterpret_cast< QUrl*>(_a[0]) = _r; }  break;
        case 8: _t->doSave((*reinterpret_cast< const QUrl(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 9: _t->doSave((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 10: _t->doSave(); break;
        case 11: _t->doSaveAs((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 12: _t->doSaveAs(); break;
        case 13: _t->doCopyToClipboard(); break;
        case 14: _t->doPrint((*reinterpret_cast< QPrinter*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (ExportManager::*_t)(const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ExportManager::errorMessage)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (ExportManager::*_t)(const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ExportManager::saveLocationChanged)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (ExportManager::*_t)(const QPixmap & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ExportManager::pixmapChanged)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (ExportManager::*_t)(const QUrl & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ExportManager::imageSaved)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (ExportManager::*_t)(const QUrl & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ExportManager::forceNotify)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        ExportManager *_t = static_cast<ExportManager *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->saveLocation(); break;
        case 1: *reinterpret_cast< QPixmap*>(_v) = _t->pixmap(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        ExportManager *_t = static_cast<ExportManager *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSaveLocation(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

const QMetaObject ExportManager::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_ExportManager.data,
      qt_meta_data_ExportManager,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *ExportManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportManager::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_ExportManager.stringdata0))
        return static_cast<void*>(const_cast< ExportManager*>(this));
    return QObject::qt_metacast(_clname);
}

int ExportManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
#ifndef QT_NO_PROPERTIES
   else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void ExportManager::errorMessage(const QString & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ExportManager::saveLocationChanged(const QString & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ExportManager::pixmapChanged(const QPixmap & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ExportManager::imageSaved(const QUrl & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ExportManager::forceNotify(const QUrl & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_END_MOC_NAMESPACE
