/****************************************************************************
** Meta object code from reading C++ file 'QuickEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/QuickEditor/QuickEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QuickEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_QuickEditor_t {
    QByteArrayData data[11];
    char stringdata0[90];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickEditor_t qt_meta_stringdata_QuickEditor = {
    {
QT_MOC_LITERAL(0, 0, 11), // "QuickEditor"
QT_MOC_LITERAL(1, 12, 8), // "grabDone"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 6), // "pixmap"
QT_MOC_LITERAL(4, 29, 10), // "cropRegion"
QT_MOC_LITERAL(5, 40, 13), // "grabCancelled"
QT_MOC_LITERAL(6, 54, 18), // "acceptImageHandler"
QT_MOC_LITERAL(7, 73, 1), // "x"
QT_MOC_LITERAL(8, 75, 1), // "y"
QT_MOC_LITERAL(9, 77, 5), // "width"
QT_MOC_LITERAL(10, 83, 6) // "height"

    },
    "QuickEditor\0grabDone\0\0pixmap\0cropRegion\0"
    "grabCancelled\0acceptImageHandler\0x\0y\0"
    "width\0height"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickEditor[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x06 /* Public */,
       5,    0,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    4,   35,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QPixmap, QMetaType::QRect,    3,    4,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    7,    8,    9,   10,

       0        // eod
};

void QuickEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        QuickEditor *_t = static_cast<QuickEditor *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->grabDone((*reinterpret_cast< const QPixmap(*)>(_a[1])),(*reinterpret_cast< const QRect(*)>(_a[2]))); break;
        case 1: _t->grabCancelled(); break;
        case 2: _t->acceptImageHandler((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (QuickEditor::*_t)(const QPixmap & , const QRect & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&QuickEditor::grabDone)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (QuickEditor::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&QuickEditor::grabCancelled)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject QuickEditor::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_QuickEditor.data,
      qt_meta_data_QuickEditor,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *QuickEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_QuickEditor.stringdata0))
        return static_cast<void*>(const_cast< QuickEditor*>(this));
    return QObject::qt_metacast(_clname);
}

int QuickEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QuickEditor::grabDone(const QPixmap & _t1, const QRect & _t2)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QuickEditor::grabCancelled()
{
    QMetaObject::activate(this, &staticMetaObject, 1, Q_NULLPTR);
}
QT_END_MOC_NAMESPACE
