/****************************************************************************
** Meta object code from reading C++ file 'SpectacleDBusAdapter.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/SpectacleDBusAdapter.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SpectacleDBusAdapter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_SpectacleDBusAdapter_t {
    QByteArrayData data[18];
    char stringdata0[1307];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SpectacleDBusAdapter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SpectacleDBusAdapter_t qt_meta_stringdata_SpectacleDBusAdapter = {
    {
QT_MOC_LITERAL(0, 0, 20), // "SpectacleDBusAdapter"
QT_MOC_LITERAL(1, 21, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 37, 17), // "org.kde.Spectacle"
QT_MOC_LITERAL(3, 55, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 75, 1048), // "  <interface name=\"org.kde.S..."
QT_MOC_LITERAL(5, 1032, 15), // "ScreenshotTaken"
QT_MOC_LITERAL(6, 1048, 0), // ""
QT_MOC_LITERAL(7, 1049, 8), // "fileName"
QT_MOC_LITERAL(8, 1058, 16), // "ScreenshotFailed"
QT_MOC_LITERAL(9, 1075, 10), // "StartAgent"
QT_MOC_LITERAL(10, 1086, 9), // "Q_NOREPLY"
QT_MOC_LITERAL(11, 1096, 10), // "FullScreen"
QT_MOC_LITERAL(12, 1107, 19), // "includeMousePointer"
QT_MOC_LITERAL(13, 1127, 13), // "CurrentScreen"
QT_MOC_LITERAL(14, 1141, 12), // "ActiveWindow"
QT_MOC_LITERAL(15, 1154, 24), // "includeWindowDecorations"
QT_MOC_LITERAL(16, 1179, 17), // "WindowUnderCursor"
QT_MOC_LITERAL(17, 1197, 17) // "RectangularRegion"

    },
    "SpectacleDBusAdapter\0D-Bus Interface\0"
    "org.kde.Spectacle\0D-Bus Introspection\0"
    "  <interface name=\"org.kde.Spectacle\">\n    <method name=\"StartAgen"
    "t\">\n    </method>\n    <method name=\"FullScreen\">\n      <arg dire"
    "ction=\"in\" type=\"b\" name=\"includeMousePointer\"/>\n    </method>\n"
    "    <method name=\"CurrentScreen\">\n      <arg direction=\"in\" type="
    "\"b\" name=\"includeMousePointer\"/>\n    </method>\n    <method name="
    "\"ActiveWindow\">\n      <arg direction=\"in\" type=\"b\" name=\"inclu"
    "deWindowDecorations\"/>\n      <arg direction=\"in\" type=\"b\" name=\""
    "includeMousePointer\"/>\n    </method>\n    <method name=\"WindowUnder"
    "Cursor\">\n      <arg direction=\"in\" type=\"b\" name=\"includeWindow"
    "Decorations\"/>\n      <arg direction=\"in\" type=\"b\" name=\"include"
    "MousePointer\"/>\n    </method>\n    <method name=\"RectangularRegion\""
    ">\n      <arg direction=\"in\" type=\"b\" name=\"includeMousePointer\""
    "/>\n    </method>\n    <signal name=\"ScreenshotTaken\">\n      <arg d"
    "irection=\"out\" type=\"s\" name=\"fileName\"/>\n    </signal>\n    <s"
    "ignal name=\"ScreenshotFailed\">\n    </signal>\n  </interface>\n\0"
    "ScreenshotTaken\0\0fileName\0ScreenshotFailed\0"
    "StartAgent\0Q_NOREPLY\0FullScreen\0"
    "includeMousePointer\0CurrentScreen\0"
    "ActiveWindow\0includeWindowDecorations\0"
    "WindowUnderCursor\0RectangularRegion"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SpectacleDBusAdapter[] = {

 // content:
       7,       // revision
       0,       // classname
       2,   14, // classinfo
       8,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    1,   58,    6, 0x06 /* Public */,
       8,    0,   61,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   62,   10, 0x0a /* Public */,
      11,    1,   63,   10, 0x0a /* Public */,
      13,    1,   66,   10, 0x0a /* Public */,
      14,    2,   69,   10, 0x0a /* Public */,
      16,    2,   74,   10, 0x0a /* Public */,
      17,    1,   79,   10, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,   15,   12,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,   15,   12,
    QMetaType::Void, QMetaType::Bool,   12,

       0        // eod
};

void SpectacleDBusAdapter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        SpectacleDBusAdapter *_t = static_cast<SpectacleDBusAdapter *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->ScreenshotTaken((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->ScreenshotFailed(); break;
        case 2: _t->StartAgent(); break;
        case 3: _t->FullScreen((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->CurrentScreen((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->ActiveWindow((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->WindowUnderCursor((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->RectangularRegion((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (SpectacleDBusAdapter::*_t)(const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleDBusAdapter::ScreenshotTaken)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (SpectacleDBusAdapter::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleDBusAdapter::ScreenshotFailed)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject SpectacleDBusAdapter::staticMetaObject = {
    { &QDBusAbstractAdaptor::staticMetaObject, qt_meta_stringdata_SpectacleDBusAdapter.data,
      qt_meta_data_SpectacleDBusAdapter,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *SpectacleDBusAdapter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpectacleDBusAdapter::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_SpectacleDBusAdapter.stringdata0))
        return static_cast<void*>(const_cast< SpectacleDBusAdapter*>(this));
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int SpectacleDBusAdapter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void SpectacleDBusAdapter::ScreenshotTaken(const QString & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SpectacleDBusAdapter::ScreenshotFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, Q_NULLPTR);
}
QT_END_MOC_NAMESPACE
