/****************************************************************************
** Meta object code from reading C++ file 'ImageGrabber.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/PlatformBackends/ImageGrabber.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ImageGrabber.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_ImageGrabber_t {
    QByteArrayData data[21];
    char stringdata0[301];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ImageGrabber_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ImageGrabber_t qt_meta_stringdata_ImageGrabber = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ImageGrabber"
QT_MOC_LITERAL(1, 13, 13), // "pixmapChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 6), // "pixmap"
QT_MOC_LITERAL(4, 35, 15), // "imageGrabFailed"
QT_MOC_LITERAL(5, 51, 21), // "capturePointerChanged"
QT_MOC_LITERAL(6, 73, 14), // "capturePointer"
QT_MOC_LITERAL(7, 88, 25), // "captureDecorationsChanged"
QT_MOC_LITERAL(8, 114, 18), // "captureDecorations"
QT_MOC_LITERAL(9, 133, 15), // "grabModeChanged"
QT_MOC_LITERAL(10, 149, 8), // "GrabMode"
QT_MOC_LITERAL(11, 158, 8), // "grabMode"
QT_MOC_LITERAL(12, 167, 11), // "doImageGrab"
QT_MOC_LITERAL(13, 179, 13), // "doOnClickGrab"
QT_MOC_LITERAL(14, 193, 13), // "InvalidChoice"
QT_MOC_LITERAL(15, 207, 10), // "FullScreen"
QT_MOC_LITERAL(16, 218, 13), // "CurrentScreen"
QT_MOC_LITERAL(17, 232, 12), // "ActiveWindow"
QT_MOC_LITERAL(18, 245, 17), // "WindowUnderCursor"
QT_MOC_LITERAL(19, 263, 19), // "TransientWithParent"
QT_MOC_LITERAL(20, 283, 17) // "RectangularRegion"

    },
    "ImageGrabber\0pixmapChanged\0\0pixmap\0"
    "imageGrabFailed\0capturePointerChanged\0"
    "capturePointer\0captureDecorationsChanged\0"
    "captureDecorations\0grabModeChanged\0"
    "GrabMode\0grabMode\0doImageGrab\0"
    "doOnClickGrab\0InvalidChoice\0FullScreen\0"
    "CurrentScreen\0ActiveWindow\0WindowUnderCursor\0"
    "TransientWithParent\0RectangularRegion"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ImageGrabber[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       4,   64, // properties
       1,   80, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       4,    0,   52,    2, 0x06 /* Public */,
       5,    1,   53,    2, 0x06 /* Public */,
       7,    1,   56,    2, 0x06 /* Public */,
       9,    1,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,   62,    2, 0x0a /* Public */,
      13,    0,   63,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QPixmap,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, 0x80000000 | 10,   11,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       3, QMetaType::QPixmap, 0x00495001,
       6, QMetaType::Bool, 0x00495103,
       8, QMetaType::Bool, 0x00495103,
      11, 0x80000000 | 10, 0x0049510b,

 // properties: notify_signal_id
       0,
       2,
       3,
       4,

 // enums: name, flags, count, data
      10, 0x0,    7,   84,

 // enum data: key, value
      14, uint(ImageGrabber::InvalidChoice),
      15, uint(ImageGrabber::FullScreen),
      16, uint(ImageGrabber::CurrentScreen),
      17, uint(ImageGrabber::ActiveWindow),
      18, uint(ImageGrabber::WindowUnderCursor),
      19, uint(ImageGrabber::TransientWithParent),
      20, uint(ImageGrabber::RectangularRegion),

       0        // eod
};

void ImageGrabber::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        ImageGrabber *_t = static_cast<ImageGrabber *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->pixmapChanged((*reinterpret_cast< const QPixmap(*)>(_a[1]))); break;
        case 1: _t->imageGrabFailed(); break;
        case 2: _t->capturePointerChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->captureDecorationsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->grabModeChanged((*reinterpret_cast< GrabMode(*)>(_a[1]))); break;
        case 5: _t->doImageGrab(); break;
        case 6: _t->doOnClickGrab(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (ImageGrabber::*_t)(const QPixmap & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ImageGrabber::pixmapChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (ImageGrabber::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ImageGrabber::imageGrabFailed)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (ImageGrabber::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ImageGrabber::capturePointerChanged)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (ImageGrabber::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ImageGrabber::captureDecorationsChanged)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (ImageGrabber::*_t)(GrabMode );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&ImageGrabber::grabModeChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        ImageGrabber *_t = static_cast<ImageGrabber *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPixmap*>(_v) = _t->pixmap(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->capturePointer(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->captureDecorations(); break;
        case 3: *reinterpret_cast< GrabMode*>(_v) = _t->grabMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        ImageGrabber *_t = static_cast<ImageGrabber *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setCapturePointer(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setCaptureDecorations(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setGrabMode(*reinterpret_cast< GrabMode*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

const QMetaObject ImageGrabber::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_ImageGrabber.data,
      qt_meta_data_ImageGrabber,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *ImageGrabber::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImageGrabber::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_ImageGrabber.stringdata0))
        return static_cast<void*>(const_cast< ImageGrabber*>(this));
    return QObject::qt_metacast(_clname);
}

int ImageGrabber::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
   else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void ImageGrabber::pixmapChanged(const QPixmap & _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ImageGrabber::imageGrabFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, Q_NULLPTR);
}

// SIGNAL 2
void ImageGrabber::capturePointerChanged(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ImageGrabber::captureDecorationsChanged(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ImageGrabber::grabModeChanged(GrabMode _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_END_MOC_NAMESPACE
