/****************************************************************************
** Meta object code from reading C++ file 'SpectacleCore.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/SpectacleCore.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SpectacleCore.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_SpectacleCore_t {
    QByteArrayData data[25];
    char stringdata0[316];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SpectacleCore_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SpectacleCore_t qt_meta_stringdata_SpectacleCore = {
    {
QT_MOC_LITERAL(0, 0, 13), // "SpectacleCore"
QT_MOC_LITERAL(1, 14, 12), // "errorMessage"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 9), // "errString"
QT_MOC_LITERAL(4, 38, 7), // "allDone"
QT_MOC_LITERAL(5, 46, 15), // "filenameChanged"
QT_MOC_LITERAL(6, 62, 8), // "filename"
QT_MOC_LITERAL(7, 71, 15), // "grabModeChanged"
QT_MOC_LITERAL(8, 87, 22), // "ImageGrabber::GrabMode"
QT_MOC_LITERAL(9, 110, 4), // "mode"
QT_MOC_LITERAL(10, 115, 10), // "grabFailed"
QT_MOC_LITERAL(11, 126, 17), // "takeNewScreenshot"
QT_MOC_LITERAL(12, 144, 7), // "timeout"
QT_MOC_LITERAL(13, 152, 14), // "includePointer"
QT_MOC_LITERAL(14, 167, 18), // "includeDecorations"
QT_MOC_LITERAL(15, 186, 16), // "showErrorMessage"
QT_MOC_LITERAL(16, 203, 17), // "screenshotUpdated"
QT_MOC_LITERAL(17, 221, 6), // "pixmap"
QT_MOC_LITERAL(18, 228, 16), // "screenshotFailed"
QT_MOC_LITERAL(19, 245, 14), // "dbusStartAgent"
QT_MOC_LITERAL(20, 260, 18), // "doStartDragAndDrop"
QT_MOC_LITERAL(21, 279, 8), // "doNotify"
QT_MOC_LITERAL(22, 288, 7), // "savedAt"
QT_MOC_LITERAL(23, 296, 10), // "doCopyPath"
QT_MOC_LITERAL(24, 307, 8) // "grabMode"

    },
    "SpectacleCore\0errorMessage\0\0errString\0"
    "allDone\0filenameChanged\0filename\0"
    "grabModeChanged\0ImageGrabber::GrabMode\0"
    "mode\0grabFailed\0takeNewScreenshot\0"
    "timeout\0includePointer\0includeDecorations\0"
    "showErrorMessage\0screenshotUpdated\0"
    "pixmap\0screenshotFailed\0dbusStartAgent\0"
    "doStartDragAndDrop\0doNotify\0savedAt\0"
    "doCopyPath\0grabMode"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SpectacleCore[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       2,  114, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    0,   82,    2, 0x06 /* Public */,
       5,    1,   83,    2, 0x06 /* Public */,
       7,    1,   86,    2, 0x06 /* Public */,
      10,    0,   89,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    4,   90,    2, 0x0a /* Public */,
      15,    1,   99,    2, 0x0a /* Public */,
      16,    1,  102,    2, 0x0a /* Public */,
      18,    0,  105,    2, 0x0a /* Public */,
      19,    0,  106,    2, 0x0a /* Public */,
      20,    0,  107,    2, 0x0a /* Public */,
      21,    1,  108,    2, 0x0a /* Public */,
      23,    1,  111,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    9,   12,   13,   14,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QPixmap,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   22,
    QMetaType::Void, QMetaType::QUrl,   22,

 // properties: name, type, flags
       6, QMetaType::QString, 0x00495103,
      24, 0x80000000 | 8, 0x0049510b,

 // properties: notify_signal_id
       2,
       3,

       0        // eod
};

void SpectacleCore::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        SpectacleCore *_t = static_cast<SpectacleCore *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->errorMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->allDone(); break;
        case 2: _t->filenameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->grabModeChanged((*reinterpret_cast< ImageGrabber::GrabMode(*)>(_a[1]))); break;
        case 4: _t->grabFailed(); break;
        case 5: _t->takeNewScreenshot((*reinterpret_cast< const ImageGrabber::GrabMode(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3])),(*reinterpret_cast< const bool(*)>(_a[4]))); break;
        case 6: _t->showErrorMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->screenshotUpdated((*reinterpret_cast< const QPixmap(*)>(_a[1]))); break;
        case 8: _t->screenshotFailed(); break;
        case 9: _t->dbusStartAgent(); break;
        case 10: _t->doStartDragAndDrop(); break;
        case 11: _t->doNotify((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 12: _t->doCopyPath((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (SpectacleCore::*_t)(const QString );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleCore::errorMessage)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (SpectacleCore::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleCore::allDone)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (SpectacleCore::*_t)(QString );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleCore::filenameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (SpectacleCore::*_t)(ImageGrabber::GrabMode );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleCore::grabModeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (SpectacleCore::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&SpectacleCore::grabFailed)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        SpectacleCore *_t = static_cast<SpectacleCore *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->filename(); break;
        case 1: *reinterpret_cast< ImageGrabber::GrabMode*>(_v) = _t->grabMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        SpectacleCore *_t = static_cast<SpectacleCore *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFilename(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setGrabMode(*reinterpret_cast< ImageGrabber::GrabMode*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject * const qt_meta_extradata_SpectacleCore[] = {
        &ImageGrabber::staticMetaObject,
    Q_NULLPTR
};

const QMetaObject SpectacleCore::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_SpectacleCore.data,
      qt_meta_data_SpectacleCore,  qt_static_metacall, qt_meta_extradata_SpectacleCore, Q_NULLPTR}
};


const QMetaObject *SpectacleCore::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpectacleCore::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_SpectacleCore.stringdata0))
        return static_cast<void*>(const_cast< SpectacleCore*>(this));
    return QObject::qt_metacast(_clname);
}

int SpectacleCore::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
   else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void SpectacleCore::errorMessage(const QString _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SpectacleCore::allDone()
{
    QMetaObject::activate(this, &staticMetaObject, 1, Q_NULLPTR);
}

// SIGNAL 2
void SpectacleCore::filenameChanged(QString _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void SpectacleCore::grabModeChanged(ImageGrabber::GrabMode _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void SpectacleCore::grabFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, Q_NULLPTR);
}
QT_END_MOC_NAMESPACE
