/****************************************************************************
** Meta object code from reading C++ file 'KSWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/Gui/KSWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KSWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KSWidget_t {
    QByteArrayData data[17];
    char stringdata0[234];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KSWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KSWidget_t qt_meta_stringdata_KSWidget = {
    {
QT_MOC_LITERAL(0, 0, 8), // "KSWidget"
QT_MOC_LITERAL(1, 9, 13), // "dragInitiated"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 20), // "newScreenshotRequest"
QT_MOC_LITERAL(4, 45, 22), // "ImageGrabber::GrabMode"
QT_MOC_LITERAL(5, 68, 4), // "mode"
QT_MOC_LITERAL(6, 73, 12), // "captureDelay"
QT_MOC_LITERAL(7, 86, 14), // "capturePointer"
QT_MOC_LITERAL(8, 101, 18), // "captureDecorations"
QT_MOC_LITERAL(9, 120, 19), // "setScreenshotPixmap"
QT_MOC_LITERAL(10, 140, 6), // "pixmap"
QT_MOC_LITERAL(11, 147, 14), // "disableOnClick"
QT_MOC_LITERAL(12, 162, 20), // "newScreenshotClicked"
QT_MOC_LITERAL(13, 183, 19), // "onClickStateChanged"
QT_MOC_LITERAL(14, 203, 5), // "state"
QT_MOC_LITERAL(15, 209, 18), // "captureModeChanged"
QT_MOC_LITERAL(16, 228, 5) // "index"

    },
    "KSWidget\0dragInitiated\0\0newScreenshotRequest\0"
    "ImageGrabber::GrabMode\0mode\0captureDelay\0"
    "capturePointer\0captureDecorations\0"
    "setScreenshotPixmap\0pixmap\0disableOnClick\0"
    "newScreenshotClicked\0onClickStateChanged\0"
    "state\0captureModeChanged\0index"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KSWidget[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    4,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,   59,    2, 0x0a /* Public */,
      11,    0,   62,    2, 0x0a /* Public */,
      12,    0,   63,    2, 0x08 /* Private */,
      13,    1,   64,    2, 0x08 /* Private */,
      15,    1,   67,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    5,    6,    7,    8,

 // slots: parameters
    QMetaType::Void, QMetaType::QPixmap,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,   16,

       0        // eod
};

void KSWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KSWidget *_t = static_cast<KSWidget *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->dragInitiated(); break;
        case 1: _t->newScreenshotRequest((*reinterpret_cast< ImageGrabber::GrabMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 2: _t->setScreenshotPixmap((*reinterpret_cast< const QPixmap(*)>(_a[1]))); break;
        case 3: _t->disableOnClick(); break;
        case 4: _t->newScreenshotClicked(); break;
        case 5: _t->onClickStateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->captureModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KSWidget::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KSWidget::dragInitiated)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KSWidget::*_t)(ImageGrabber::GrabMode , int , bool , bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KSWidget::newScreenshotRequest)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject KSWidget::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_KSWidget.data,
      qt_meta_data_KSWidget,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KSWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KSWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KSWidget.stringdata0))
        return static_cast<void*>(const_cast< KSWidget*>(this));
    return QWidget::qt_metacast(_clname);
}

int KSWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KSWidget::dragInitiated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, Q_NULLPTR);
}

// SIGNAL 1
void KSWidget::newScreenshotRequest(ImageGrabber::GrabMode _t1, int _t2, bool _t3, bool _t4)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_END_MOC_NAMESPACE
