/****************************************************************************
** Meta object code from reading C++ file 'KSMainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/spectacle-17.04.3/src/Gui/KSMainWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KSMainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KSMainWindow_t {
    QByteArrayData data[25];
    char stringdata0[339];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KSMainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KSMainWindow_t qt_meta_stringdata_KSMainWindow = {
    {
QT_MOC_LITERAL(0, 0, 12), // "KSMainWindow"
QT_MOC_LITERAL(1, 13, 20), // "newScreenshotRequest"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 22), // "ImageGrabber::GrabMode"
QT_MOC_LITERAL(4, 58, 4), // "mode"
QT_MOC_LITERAL(5, 63, 7), // "timeout"
QT_MOC_LITERAL(6, 71, 14), // "includePointer"
QT_MOC_LITERAL(7, 86, 18), // "includeDecorations"
QT_MOC_LITERAL(8, 105, 18), // "dragAndDropRequest"
QT_MOC_LITERAL(9, 124, 17), // "captureScreenshot"
QT_MOC_LITERAL(10, 142, 15), // "showPrintDialog"
QT_MOC_LITERAL(11, 158, 21), // "showPreferencesDialog"
QT_MOC_LITERAL(12, 180, 23), // "showImageSharedFeedback"
QT_MOC_LITERAL(13, 204, 5), // "error"
QT_MOC_LITERAL(14, 210, 7), // "message"
QT_MOC_LITERAL(15, 218, 15), // "sendToClipboard"
QT_MOC_LITERAL(16, 234, 4), // "init"
QT_MOC_LITERAL(17, 239, 13), // "buildSaveMenu"
QT_MOC_LITERAL(18, 253, 4), // "save"
QT_MOC_LITERAL(19, 258, 6), // "saveAs"
QT_MOC_LITERAL(20, 265, 11), // "saveAndExit"
QT_MOC_LITERAL(21, 277, 20), // "setScreenshotAndShow"
QT_MOC_LITERAL(22, 298, 6), // "pixmap"
QT_MOC_LITERAL(23, 305, 24), // "setScreenshotWindowTitle"
QT_MOC_LITERAL(24, 330, 8) // "location"

    },
    "KSMainWindow\0newScreenshotRequest\0\0"
    "ImageGrabber::GrabMode\0mode\0timeout\0"
    "includePointer\0includeDecorations\0"
    "dragAndDropRequest\0captureScreenshot\0"
    "showPrintDialog\0showPreferencesDialog\0"
    "showImageSharedFeedback\0error\0message\0"
    "sendToClipboard\0init\0buildSaveMenu\0"
    "save\0saveAs\0saveAndExit\0setScreenshotAndShow\0"
    "pixmap\0setScreenshotWindowTitle\0"
    "location"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KSMainWindow[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   84,    2, 0x06 /* Public */,
       8,    0,   93,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    4,   94,    2, 0x08 /* Private */,
      10,    0,  103,    2, 0x08 /* Private */,
      11,    0,  104,    2, 0x08 /* Private */,
      12,    2,  105,    2, 0x08 /* Private */,
      15,    0,  110,    2, 0x08 /* Private */,
      16,    0,  111,    2, 0x08 /* Private */,
      17,    0,  112,    2, 0x08 /* Private */,
      18,    0,  113,    2, 0x08 /* Private */,
      19,    0,  114,    2, 0x08 /* Private */,
      20,    0,  115,    2, 0x08 /* Private */,
      21,    1,  116,    2, 0x0a /* Public */,
      23,    1,  119,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    4,    5,    6,    7,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    4,    5,    6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::QString,   13,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPixmap,   22,
    QMetaType::Void, QMetaType::QUrl,   24,

       0        // eod
};

void KSMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KSMainWindow *_t = static_cast<KSMainWindow *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->newScreenshotRequest((*reinterpret_cast< ImageGrabber::GrabMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 1: _t->dragAndDropRequest(); break;
        case 2: _t->captureScreenshot((*reinterpret_cast< ImageGrabber::GrabMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 3: _t->showPrintDialog(); break;
        case 4: _t->showPreferencesDialog(); break;
        case 5: _t->showImageSharedFeedback((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->sendToClipboard(); break;
        case 7: _t->init(); break;
        case 8: _t->buildSaveMenu(); break;
        case 9: _t->save(); break;
        case 10: _t->saveAs(); break;
        case 11: _t->saveAndExit(); break;
        case 12: _t->setScreenshotAndShow((*reinterpret_cast< const QPixmap(*)>(_a[1]))); break;
        case 13: _t->setScreenshotWindowTitle((*reinterpret_cast< QUrl(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KSMainWindow::*_t)(ImageGrabber::GrabMode , int , bool , bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KSMainWindow::newScreenshotRequest)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KSMainWindow::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KSMainWindow::dragAndDropRequest)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject KSMainWindow::staticMetaObject = {
    { &QDialog::staticMetaObject, qt_meta_stringdata_KSMainWindow.data,
      qt_meta_data_KSMainWindow,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KSMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KSMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KSMainWindow.stringdata0))
        return static_cast<void*>(const_cast< KSMainWindow*>(this));
    return QDialog::qt_metacast(_clname);
}

int KSMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void KSMainWindow::newScreenshotRequest(ImageGrabber::GrabMode _t1, int _t2, bool _t3, bool _t4)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KSMainWindow::dragAndDropRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 1, Q_NULLPTR);
}
QT_END_MOC_NAMESPACE
